/* GNOME DB library
 * Copyright (C) 1999-2002 The GNOME Foundation.
 *
 * AUTHORS:
 *      Rodrigo Moya <rodrigo@gnome-db.org>
 *
 * This Library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This Library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this Library; see the file COPYING.LIB.  If not,
 * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <string.h>
#include <libgnomedb/gnome-db-config.h>
#include <libgnomedb/gnome-db-data-source-selector.h>
#include <libgnomedb/gnome-db-util.h>

#define PARENT_TYPE GTK_TYPE_OPTION_MENU

struct _GnomeDbDataSourceSelectorPrivate {
	guint config_listener;
};

static void gnome_db_data_source_selector_class_init (GnomeDbDataSourceSelectorClass *klass);
static void gnome_db_data_source_selector_init       (GnomeDbDataSourceSelector *selector,
						      GnomeDbDataSourceSelectorClass *klass);
static void gnome_db_data_source_selector_finalize   (GObject *object);

static GObjectClass *parent_class = NULL;

/*
 * Private functions
 */

static void
show_data_sources (GnomeDbDataSourceSelector *selector, const gchar *name)
{
	GdaDataSourceInfo *dsn_info = NULL;

	g_return_if_fail (GNOME_DB_IS_DATA_SOURCE_SELECTOR (selector));

	gtk_option_menu_remove_menu (GTK_OPTION_MENU (selector));

	if (name != NULL) {
		dsn_info = gnome_db_config_find_data_source (name);
		if (dsn_info != NULL) {
			gnome_db_option_menu_add_item (GTK_OPTION_MENU (selector),
						       dsn_info->name);
			gnome_db_option_menu_set_selection (GTK_OPTION_MENU (selector),
							    dsn_info->name);
			gnome_db_config_free_data_source_info (dsn_info);
		}
	} else {
		GList *dsn_list, *l;

		dsn_list = gnome_db_config_get_data_source_list ();
		for (l = dsn_list; l != NULL; l = l->next) {
			dsn_info = (GdaDataSourceInfo *) l->data;
			if (!dsn_info)
				continue;

			/* add the data source to the option menu */
			gnome_db_option_menu_add_item (GTK_OPTION_MENU (selector), dsn_info->name);
		}

		gnome_db_config_free_data_source_list (dsn_list);

		gtk_option_menu_set_history (GTK_OPTION_MENU (selector), 0);
	}
}

/*
 * GnomeDbDataSourceSelector class implementation
 */

static void
gnome_db_data_source_selector_class_init (GnomeDbDataSourceSelectorClass *klass)
{
	GObjectClass *object_class = G_OBJECT_CLASS (klass);

	parent_class = g_type_class_peek_parent (klass);

	object_class->finalize = gnome_db_data_source_selector_finalize;
}

static void
config_changed_cb (GConfClient *conf_client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
{
	const gchar *current_dsn;
	GnomeDbDataSourceSelector *selector = (GnomeDbDataSourceSelector *) user_data;

	g_return_if_fail (GNOME_DB_IS_DATA_SOURCE_SELECTOR (selector));

	current_dsn = g_strdup (gnome_db_option_menu_get_selection (GTK_OPTION_MENU (selector)));
	show_data_sources (selector, NULL);
	gnome_db_option_menu_set_selection (GTK_OPTION_MENU (selector), current_dsn);
	g_free (current_dsn);
}

static void
gnome_db_data_source_selector_init (GnomeDbDataSourceSelector *selector,
				    GnomeDbDataSourceSelectorClass *klass)
{
	g_return_if_fail (GNOME_DB_IS_DATA_SOURCE_SELECTOR (selector));

	/* allocate private structure */
	selector->priv = g_new0 (GnomeDbDataSourceSelectorPrivate, 1);

	/* add listener for configuration */
	selector->priv->config_listener = gnome_db_config_add_listener (
		"/apps/libgda/Datasources",
		(GConfClientNotifyFunc) config_changed_cb,
		selector);
}

static void
gnome_db_data_source_selector_finalize (GObject *object)
{
	GnomeDbDataSourceSelector *selector = (GnomeDbDataSourceSelector *) object;

	g_return_if_fail (GNOME_DB_IS_DATA_SOURCE_SELECTOR (selector));

	if (selector->priv->config_listener != -1) {
		gnome_db_config_remove_listener (selector->priv->config_listener);
		selector->priv->config_listener = -1;
	}

	/* free memory */
	g_free (selector->priv);
	selector->priv = NULL;

	parent_class->finalize (object);
}

GType
gnome_db_data_source_selector_get_type (void)
{
	static GType type = 0;

	if (!type) {
		static const GTypeInfo info = {
			sizeof (GnomeDbDataSourceSelectorClass),
			(GBaseInitFunc) NULL,
			(GBaseFinalizeFunc) NULL,
			(GClassInitFunc) gnome_db_data_source_selector_class_init,
			NULL,
			NULL,
			sizeof (GnomeDbDataSourceSelector),
			0,
			(GInstanceInitFunc) gnome_db_data_source_selector_init
		};
		type = g_type_register_static (PARENT_TYPE,
					       "GnomeDbDataSourceSelector",
					       &info, 0);
	}
	return type;
}

/**
 * gnome_db_data_source_selector_new
 * @name: name of data source to display.
 *
 * Create a new #GnomeDbDataSourceSelector, which is just a #GtkOptionMenu
 * which displays, as its menu items, all the data sources currently
 * configured in the system. It is useful for connection and configuration
 * screens, where the user has to choose a data source to work with.
 *
 * Returns: the newly created widget.
 */
GtkWidget *
gnome_db_data_source_selector_new (const gchar *name)
{
	GnomeDbDataSourceSelector *selector;

	selector = g_object_new (GNOME_DB_TYPE_DATA_SOURCE_SELECTOR, NULL);
	show_data_sources (selector, name);

	return GTK_WIDGET (selector);
}
